"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Lint = __importStar(require("tslint"));
const ts = __importStar(require("typescript"));
const array_flatten_1 = __importDefault(require("array-flatten"));
const path_1 = require("path");
class Rule extends Lint.Rules.AbstractRule {
    isEnabled() {
        return super.isEnabled() && this.ruleArguments.length > 0;
    }
    apply(sourceFile) {
        return this.applyWithFunction(sourceFile, walk, this.ruleArguments);
    }
}
Rule.metadata = {
    ruleName: 'no-named-export',
    description: 'Disallows named exports in ES6-style modules.',
    descriptionDetails: 'Use default export instead.',
    rationale: 'Can be used to mimic `module interface` with default export.',
    optionsDescription: 'A list of file names or regular expression patterns.',
    options: {
        type: 'array',
        items: {
            oneOf: [
                {
                    type: 'string',
                    minLength: 1
                },
                {
                    type: 'array',
                    items: {
                        type: 'string'
                    },
                    minLength: 1
                }
            ]
        }
    },
    optionExamples: [
        [true, '^view\\.(native|web|ios|android)\\.tsx$'],
        [true, 'state', ['^view\\w+', '^app']]
    ],
    type: 'maintainability',
    typescriptOnly: false
};
Rule.FAILURE_STRING = 'Use of named exports is forbidden';
exports.Rule = Rule;
function walk(ctx) {
    const { sourceFile } = ctx;
    if (ctx.sourceFile.isDeclarationFile || !ts.isExternalModule(ctx.sourceFile)) {
        return;
    }
    if (array_flatten_1.default(ctx.options).some(p => new RegExp(p).test(path_1.basename(sourceFile.fileName)))) {
        const syntaxList = (sourceFile.getChildren() || [])
            .find(node => node.kind === ts.SyntaxKind.SyntaxList);
        if (syntaxList !== undefined) {
            (syntaxList.getChildren() || []).forEach((node) => {
                if (node.kind === ts.SyntaxKind.ExportKeyword) {
                    ctx.addFailureAtNode(node, Rule.FAILURE_STRING);
                }
            });
        }
        sourceFile.statements.forEach((statement) => {
            const { modifiers = [] } = statement;
            if (statement.kind === ts.SyntaxKind.ExportDeclaration) {
                const { exportClause = { elements: [] } } = statement;
                if (!(exportClause.elements.length === 1 &&
                    exportClause.elements[0].name.escapedText === 'default')) {
                    ctx.addFailureAtNode(statement.getChildAt(0, sourceFile), Rule.FAILURE_STRING);
                }
            }
            else if (modifiers.length >= 1 &&
                modifiers[0].kind === ts.SyntaxKind.ExportKeyword &&
                (modifiers.length < 2 || modifiers[1].kind !== ts.SyntaxKind.DefaultKeyword)) {
                ctx.addFailureAtNode(statement.getChildAt(0, sourceFile), Rule.FAILURE_STRING);
            }
        });
    }
}
