"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Lint = __importStar(require("tslint"));
const ts = __importStar(require("typescript"));
const array_flatten_1 = __importDefault(require("array-flatten"));
class Rule extends Lint.Rules.AbstractRule {
    isEnabled() {
        return super.isEnabled() && this.ruleArguments.length > 0;
    }
    apply(sourceFile) {
        return this.applyWithFunction(sourceFile, walk, this.ruleArguments);
    }
}
Rule.metadata = {
    ruleName: 'no-named-import',
    description: 'Disallows named imports in ES6-style modules.',
    descriptionDetails: 'Use default import instead.',
    rationale: 'Can be used to mimic `module interface` with default import.',
    optionsDescription: 'A list of module names or regular expression patterns.',
    options: {
        type: 'array',
        items: {
            oneOf: [
                {
                    type: 'string',
                    minLength: 1
                },
                {
                    type: 'array',
                    items: {
                        type: 'string'
                    },
                    minLength: 1
                }
            ]
        }
    },
    optionExamples: [
        [true, '^view$'],
        [true, 'state', ['^view\\w+', '^app']]
    ],
    type: 'maintainability',
    typescriptOnly: false
};
Rule.FAILURE_STRING = 'Use of named imports is forbidden';
exports.Rule = Rule;
function walk(ctx) {
    const { sourceFile } = ctx;
    const { statements } = sourceFile;
    const moduleChecks = array_flatten_1.default(ctx.options).map(p => new RegExp(p));
    statements.forEach((statement, idx) => {
        if (statement.kind === ts.SyntaxKind.ImportDeclaration) {
            const { importClause = { namedBindings: undefined }, moduleSpecifier } = statement;
            if (importClause.namedBindings !== undefined && (moduleSpecifier.kind === ts.SyntaxKind.StringLiteral &&
                moduleChecks.some(p => p.test(moduleSpecifier.text))) || (moduleSpecifier.kind === ts.SyntaxKind.BinaryExpression &&
                statements.length >= (idx + 3) &&
                statements[idx + 2].kind === ts.SyntaxKind.ExpressionStatement &&
                moduleChecks
                    .some(p => p.test(statements[idx + 2].expression.getText())))) {
                ctx.addFailureAtNode(statement.getChildAt(0, sourceFile), Rule.FAILURE_STRING);
            }
        }
    });
}
